class PackageNotFoundException(Exception):
    pass


class KeyNotFoundException(Exception):
    pass


class UpdateDBException(Exception):
    pass


class TableNotFoundException(Exception):
    pass


class DBModuleNotFoundException(Exception):
    pass


class ModuleNotFoundException(Exception):
    pass
