class UpdateEvent(object):
    ACCESS_TOKEN_SUCCESS = "access_token_success"
    UPDATE_ACCESS_TOKEN = "update_access_token"
    ACCESS_TOKEN_UNAUTHORIZED = "access_token_unauthorized"
    ACCESS_TOKEN_FAILED = "access_token_failed"
    RECONNECTING = "access_token_failed"
    CLOSE_SESSION = "close_session"
    REFRESH_TOKEN_SUCCESS = "refresh_token_success"
    REFRESH_TOKEN_FAILED = "refresh_token_failed"
    REFRESH_TOKEN_BAD = "refresh_token_bad"
    REFRESH_TOKEN_EXPIRED = "refresh_token_expired"
    AUTHENTICATION_SUCCESS = "authentication_success"
    AUTHENTICATION_FAILED = "authentication_failed"
